### ms-lrec.idl
"""
RPC definitions for the following interfaces:
- NetEventForwarder (v1.0): 22e5386d-8b12-4bf0-b0ec-6a1ea419e366
This file is auto-generated by midl-to-scapy, do not modify.
"""

import uuid


from scapy.layers.dcerpc import (
    NDRPacket,
    DceRpcOp,
    NDRConfStrLenField,
    NDRConfVarStrNullField,
    NDRConfVarStrNullFieldUtf16,
    NDRContextHandle,
    NDRFullPointerField,
    NDRIntField,
    NDRPacketField,
    register_dcerpc_interface,
)


class RpcNetEventOpenSession_Request(NDRPacket):
    fields_desc = [NDRConfVarStrNullFieldUtf16("LoggerName", "")]


class RpcNetEventOpenSession_Response(NDRPacket):
    fields_desc = [
        NDRPacketField("SessionHandle", NDRContextHandle(), NDRContextHandle),
        NDRIntField("status", 0),
    ]


class EVENT_BUFFER(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRIntField("BufferLength", None, size_of="Buffer"),
        NDRFullPointerField(
            NDRConfStrLenField("Buffer", "", size_is=lambda pkt: pkt.BufferLength),
            deferred=True,
        ),
    ]


class RpcNetEventReceiveData_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("SessionHandle", NDRContextHandle(), NDRContextHandle)
    ]


class RpcNetEventReceiveData_Response(NDRPacket):
    fields_desc = [
        NDRPacketField("EventBuffer", EVENT_BUFFER(), EVENT_BUFFER),
        NDRIntField("status", 0),
    ]


class RpcNetEventCloseSession_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("SessionHandle", NDRContextHandle(), NDRContextHandle)
    ]


class RpcNetEventCloseSession_Response(NDRPacket):
    fields_desc = [
        NDRPacketField("SessionHandle", NDRContextHandle(), NDRContextHandle),
        NDRIntField("status", 0),
    ]


NETEVENTFORWARDER_OPNUMS = {
    0: DceRpcOp(RpcNetEventOpenSession_Request, RpcNetEventOpenSession_Response),
    1: DceRpcOp(RpcNetEventReceiveData_Request, RpcNetEventReceiveData_Response),
    2: DceRpcOp(RpcNetEventCloseSession_Request, RpcNetEventCloseSession_Response),
}
register_dcerpc_interface(
    name="NetEventForwarder",
    uuid=uuid.UUID("22e5386d-8b12-4bf0-b0ec-6a1ea419e366"),
    version="1.0",
    opnums=NETEVENTFORWARDER_OPNUMS,
)



### ms-lsat.idl
"""
RPC definitions for the following interfaces:
- lsarpc (v0.0): 12345778-1234-ABCD-EF00-0123456789AB
This file is auto-generated by midl-to-scapy, do not modify.
"""

from enum import IntEnum
import uuid

from scapy.fields import StrFixedLenField
from scapy.layers.dcerpc import (
    NDRPacket,
    DceRpcOp,
    NDRByteField,
    NDRConfFieldListField,
    NDRConfPacketListField,
    NDRConfStrLenField,
    NDRConfVarStrLenField,
    NDRConfVarStrLenFieldUtf16,
    NDRConfVarStrNullField,
    NDRConfVarStrNullFieldUtf16,
    NDRContextHandle,
    NDRFullPointerField,
    NDRInt3264EnumField,
    NDRIntField,
    NDRPacketField,
    NDRShortField,
    NDRSignedIntField,
    register_dcerpc_interface,
)


class LsarClose_Request(NDRPacket):
    fields_desc = [NDRPacketField("ObjectHandle", NDRContextHandle(), NDRContextHandle)]


class LsarClose_Response(NDRPacket):
    fields_desc = [
        NDRPacketField("ObjectHandle", NDRContextHandle(), NDRContextHandle),
        NDRIntField("status", 0),
    ]


class PSTRING(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRShortField("Length", None, size_of="Buffer"),
        NDRShortField("MaximumLength", None, size_of="Buffer"),
        NDRFullPointerField(
            NDRConfVarStrLenField(
                "Buffer",
                "",
                size_is=lambda pkt: pkt.MaximumLength,
                length_is=lambda pkt: pkt.Length,
            ),
            deferred=True,
        ),
    ]


class RPC_SID_IDENTIFIER_AUTHORITY(NDRPacket):
    fields_desc = [StrFixedLenField("Value", "", length=6)]


class PRPC_SID(NDRPacket):
    ALIGNMENT = (4, 8)
    DEPORTED_CONFORMANTS = ["SubAuthority"]
    fields_desc = [
        NDRByteField("Revision", 0),
        NDRByteField("SubAuthorityCount", None, size_of="SubAuthority"),
        NDRPacketField(
            "IdentifierAuthority",
            RPC_SID_IDENTIFIER_AUTHORITY(),
            RPC_SID_IDENTIFIER_AUTHORITY,
        ),
        NDRConfFieldListField(
            "SubAuthority",
            [],
            NDRIntField("", 0),
            size_is=lambda pkt: pkt.SubAuthorityCount,
            conformant_in_struct=True,
        ),
    ]


class PLSAPR_ACL(NDRPacket):
    ALIGNMENT = (2, 2)
    DEPORTED_CONFORMANTS = ["Dummy1"]
    fields_desc = [
        NDRByteField("AclRevision", 0),
        NDRByteField("Sbz1", 0),
        NDRShortField("AclSize", None, size_of="Dummy1", adjust=lambda _, x: (x + 4)),
        NDRConfStrLenField(
            "Dummy1",
            "",
            size_is=lambda pkt: (pkt.AclSize - 4),
            conformant_in_struct=True,
        ),
    ]


class PLSAPR_SECURITY_DESCRIPTOR(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRByteField("Revision", 0),
        NDRByteField("Sbz1", 0),
        NDRShortField("Control", 0),
        NDRFullPointerField(
            NDRPacketField("Owner", PRPC_SID(), PRPC_SID), deferred=True
        ),
        NDRFullPointerField(
            NDRPacketField("Group", PRPC_SID(), PRPC_SID), deferred=True
        ),
        NDRFullPointerField(
            NDRPacketField("Sacl", PLSAPR_ACL(), PLSAPR_ACL), deferred=True
        ),
        NDRFullPointerField(
            NDRPacketField("Dacl", PLSAPR_ACL(), PLSAPR_ACL), deferred=True
        ),
    ]


class SECURITY_IMPERSONATION_LEVEL(IntEnum):
    SecurityAnonymous = 0
    SecurityIdentification = 1
    SecurityImpersonation = 2
    SecurityDelegation = 3


class PSECURITY_QUALITY_OF_SERVICE(NDRPacket):
    ALIGNMENT = (4, 4)
    fields_desc = [
        NDRIntField("Length", 0),
        NDRInt3264EnumField("ImpersonationLevel", 0, SECURITY_IMPERSONATION_LEVEL),
        NDRByteField("ContextTrackingMode", 0),
        NDRByteField("EffectiveOnly", 0),
    ]


class PLSAPR_OBJECT_ATTRIBUTES(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRIntField("Length", 0),
        NDRFullPointerField(NDRByteField("RootDirectory", 0), deferred=True),
        NDRFullPointerField(
            NDRPacketField("ObjectName", PSTRING(), PSTRING), deferred=True
        ),
        NDRIntField("Attributes", 0),
        NDRFullPointerField(
            NDRPacketField(
                "SecurityDescriptor",
                PLSAPR_SECURITY_DESCRIPTOR(),
                PLSAPR_SECURITY_DESCRIPTOR,
            ),
            deferred=True,
        ),
        NDRFullPointerField(
            NDRPacketField(
                "SecurityQualityOfService",
                PSECURITY_QUALITY_OF_SERVICE(),
                PSECURITY_QUALITY_OF_SERVICE,
            ),
            deferred=True,
        ),
    ]


class LsarOpenPolicy_Request(NDRPacket):
    fields_desc = [
        NDRFullPointerField(NDRShortField("SystemName", 0)),
        NDRPacketField(
            "ObjectAttributes", PLSAPR_OBJECT_ATTRIBUTES(), PLSAPR_OBJECT_ATTRIBUTES
        ),
        NDRIntField("DesiredAccess", 0),
    ]


class LsarOpenPolicy_Response(NDRPacket):
    fields_desc = [
        NDRPacketField("PolicyHandle", NDRContextHandle(), NDRContextHandle),
        NDRIntField("status", 0),
    ]


class PRPC_UNICODE_STRING(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRShortField("Length", None, size_of="Buffer", adjust=lambda _, x: (x * 2)),
        NDRShortField(
            "MaximumLength", None, size_of="Buffer", adjust=lambda _, x: (x * 2)
        ),
        NDRFullPointerField(
            NDRConfVarStrLenFieldUtf16(
                "Buffer",
                "",
                size_is=lambda pkt: (pkt.MaximumLength // 2),
                length_is=lambda pkt: (pkt.Length // 2),
            ),
            deferred=True,
        ),
    ]


class SID_NAME_USE(IntEnum):
    SidTypeUser = 1
    SidTypeGroup = 2
    SidTypeDomain = 3
    SidTypeAlias = 4
    SidTypeWellKnownGroup = 5
    SidTypeDeletedAccount = 6
    SidTypeInvalid = 7
    SidTypeUnknown = 8
    SidTypeComputer = 9
    SidTypeLabel = 10


class PLSA_TRANSLATED_SID(NDRPacket):
    ALIGNMENT = (4, 4)
    fields_desc = [
        NDRInt3264EnumField("Use", 0, SID_NAME_USE),
        NDRIntField("RelativeId", 0),
        NDRSignedIntField("DomainIndex", 0),
    ]


class PLSAPR_TRANSLATED_SIDS(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRIntField("Entries", None, size_of="Sids"),
        NDRFullPointerField(
            NDRConfPacketListField(
                "Sids",
                [PLSA_TRANSLATED_SID()],
                PLSA_TRANSLATED_SID,
                size_is=lambda pkt: pkt.Entries,
            ),
            deferred=True,
        ),
    ]


class LSAP_LOOKUP_LEVEL(IntEnum):
    LsapLookupWksta = 1
    LsapLookupPDC = 2
    LsapLookupTDL = 3
    LsapLookupGC = 4
    LsapLookupXForestReferral = 5
    LsapLookupXForestResolve = 6
    LsapLookupRODCReferralToFullDC = 7


class RPC_UNICODE_STRING(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRShortField("Length", None, size_of="Buffer", adjust=lambda _, x: (x * 2)),
        NDRShortField(
            "MaximumLength", None, size_of="Buffer", adjust=lambda _, x: (x * 2)
        ),
        NDRFullPointerField(
            NDRConfVarStrLenFieldUtf16(
                "Buffer",
                "",
                size_is=lambda pkt: (pkt.MaximumLength // 2),
                length_is=lambda pkt: (pkt.Length // 2),
            ),
            deferred=True,
        ),
    ]


class PLSAPR_TRUST_INFORMATION(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRPacketField("Name", RPC_UNICODE_STRING(), RPC_UNICODE_STRING),
        NDRFullPointerField(NDRPacketField("Sid", PRPC_SID(), PRPC_SID), deferred=True),
    ]


class PLSAPR_REFERENCED_DOMAIN_LIST(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRIntField("Entries", None, size_of="Domains"),
        NDRFullPointerField(
            NDRConfPacketListField(
                "Domains",
                [PLSAPR_TRUST_INFORMATION()],
                PLSAPR_TRUST_INFORMATION,
                size_is=lambda pkt: pkt.Entries,
            ),
            deferred=True,
        ),
        NDRIntField("MaxEntries", 0),
    ]


class LsarLookupNames_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("PolicyHandle", NDRContextHandle(), NDRContextHandle),
        NDRIntField("Count", None, size_of="Names"),
        NDRConfPacketListField(
            "Names",
            [PRPC_UNICODE_STRING()],
            PRPC_UNICODE_STRING,
            size_is=lambda pkt: pkt.Count,
        ),
        NDRPacketField(
            "TranslatedSids", PLSAPR_TRANSLATED_SIDS(), PLSAPR_TRANSLATED_SIDS
        ),
        NDRInt3264EnumField("LookupLevel", 0, LSAP_LOOKUP_LEVEL),
        NDRIntField("MappedCount", 0),
    ]


class LsarLookupNames_Response(NDRPacket):
    fields_desc = [
        NDRFullPointerField(
            NDRPacketField(
                "ReferencedDomains",
                PLSAPR_REFERENCED_DOMAIN_LIST(),
                PLSAPR_REFERENCED_DOMAIN_LIST,
            )
        ),
        NDRPacketField(
            "TranslatedSids", PLSAPR_TRANSLATED_SIDS(), PLSAPR_TRANSLATED_SIDS
        ),
        NDRIntField("MappedCount", 0),
        NDRIntField("status", 0),
    ]


class PLSAPR_SID_INFORMATION(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRFullPointerField(NDRPacketField("Sid", PRPC_SID(), PRPC_SID), deferred=True)
    ]


class PLSAPR_SID_ENUM_BUFFER(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRIntField("Entries", None, size_of="SidInfo"),
        NDRFullPointerField(
            NDRConfPacketListField(
                "SidInfo",
                [PLSAPR_SID_INFORMATION()],
                PLSAPR_SID_INFORMATION,
                size_is=lambda pkt: pkt.Entries,
            ),
            deferred=True,
        ),
    ]


class PLSAPR_TRANSLATED_NAME(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRInt3264EnumField("Use", 0, SID_NAME_USE),
        NDRPacketField("Name", RPC_UNICODE_STRING(), RPC_UNICODE_STRING),
        NDRSignedIntField("DomainIndex", 0),
    ]


class PLSAPR_TRANSLATED_NAMES(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRIntField("Entries", None, size_of="Names"),
        NDRFullPointerField(
            NDRConfPacketListField(
                "Names",
                [PLSAPR_TRANSLATED_NAME()],
                PLSAPR_TRANSLATED_NAME,
                size_is=lambda pkt: pkt.Entries,
            ),
            deferred=True,
        ),
    ]


class LsarLookupSids_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("PolicyHandle", NDRContextHandle(), NDRContextHandle),
        NDRPacketField(
            "SidEnumBuffer", PLSAPR_SID_ENUM_BUFFER(), PLSAPR_SID_ENUM_BUFFER
        ),
        NDRPacketField(
            "TranslatedNames", PLSAPR_TRANSLATED_NAMES(), PLSAPR_TRANSLATED_NAMES
        ),
        NDRInt3264EnumField("LookupLevel", 0, LSAP_LOOKUP_LEVEL),
        NDRIntField("MappedCount", 0),
    ]


class LsarLookupSids_Response(NDRPacket):
    fields_desc = [
        NDRFullPointerField(
            NDRPacketField(
                "ReferencedDomains",
                PLSAPR_REFERENCED_DOMAIN_LIST(),
                PLSAPR_REFERENCED_DOMAIN_LIST,
            )
        ),
        NDRPacketField(
            "TranslatedNames", PLSAPR_TRANSLATED_NAMES(), PLSAPR_TRANSLATED_NAMES
        ),
        NDRIntField("MappedCount", 0),
        NDRIntField("status", 0),
    ]


class LsarOpenPolicy2_Request(NDRPacket):
    fields_desc = [
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("SystemName", "")),
        NDRPacketField(
            "ObjectAttributes", PLSAPR_OBJECT_ATTRIBUTES(), PLSAPR_OBJECT_ATTRIBUTES
        ),
        NDRIntField("DesiredAccess", 0),
    ]


class LsarOpenPolicy2_Response(NDRPacket):
    fields_desc = [
        NDRPacketField("PolicyHandle", NDRContextHandle(), NDRContextHandle),
        NDRIntField("status", 0),
    ]


class LsarGetUserName_Request(NDRPacket):
    fields_desc = [
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("SystemName", "")),
        NDRFullPointerField(
            NDRPacketField("UserName", PRPC_UNICODE_STRING(), PRPC_UNICODE_STRING)
        ),
        NDRFullPointerField(
            NDRFullPointerField(
                NDRPacketField("DomainName", PRPC_UNICODE_STRING(), PRPC_UNICODE_STRING)
            )
        ),
    ]


class LsarGetUserName_Response(NDRPacket):
    fields_desc = [
        NDRFullPointerField(
            NDRPacketField("UserName", PRPC_UNICODE_STRING(), PRPC_UNICODE_STRING)
        ),
        NDRFullPointerField(
            NDRFullPointerField(
                NDRPacketField("DomainName", PRPC_UNICODE_STRING(), PRPC_UNICODE_STRING)
            )
        ),
        NDRIntField("status", 0),
    ]


class PLSAPR_TRANSLATED_NAME_EX(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRInt3264EnumField("Use", 0, SID_NAME_USE),
        NDRPacketField("Name", RPC_UNICODE_STRING(), RPC_UNICODE_STRING),
        NDRSignedIntField("DomainIndex", 0),
        NDRIntField("Flags", 0),
    ]


class PLSAPR_TRANSLATED_NAMES_EX(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRIntField("Entries", None, size_of="Names"),
        NDRFullPointerField(
            NDRConfPacketListField(
                "Names",
                [PLSAPR_TRANSLATED_NAME_EX()],
                PLSAPR_TRANSLATED_NAME_EX,
                size_is=lambda pkt: pkt.Entries,
            ),
            deferred=True,
        ),
    ]


class LsarLookupSids2_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("PolicyHandle", NDRContextHandle(), NDRContextHandle),
        NDRPacketField(
            "SidEnumBuffer", PLSAPR_SID_ENUM_BUFFER(), PLSAPR_SID_ENUM_BUFFER
        ),
        NDRPacketField(
            "TranslatedNames", PLSAPR_TRANSLATED_NAMES_EX(), PLSAPR_TRANSLATED_NAMES_EX
        ),
        NDRInt3264EnumField("LookupLevel", 0, LSAP_LOOKUP_LEVEL),
        NDRIntField("MappedCount", 0),
        NDRIntField("LookupOptions", 0),
        NDRIntField("ClientRevision", 0),
    ]


class LsarLookupSids2_Response(NDRPacket):
    fields_desc = [
        NDRFullPointerField(
            NDRPacketField(
                "ReferencedDomains",
                PLSAPR_REFERENCED_DOMAIN_LIST(),
                PLSAPR_REFERENCED_DOMAIN_LIST,
            )
        ),
        NDRPacketField(
            "TranslatedNames", PLSAPR_TRANSLATED_NAMES_EX(), PLSAPR_TRANSLATED_NAMES_EX
        ),
        NDRIntField("MappedCount", 0),
        NDRIntField("status", 0),
    ]


class PLSAPR_TRANSLATED_SID_EX(NDRPacket):
    ALIGNMENT = (4, 4)
    fields_desc = [
        NDRInt3264EnumField("Use", 0, SID_NAME_USE),
        NDRIntField("RelativeId", 0),
        NDRSignedIntField("DomainIndex", 0),
        NDRIntField("Flags", 0),
    ]


class PLSAPR_TRANSLATED_SIDS_EX(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRIntField("Entries", None, size_of="Sids"),
        NDRFullPointerField(
            NDRConfPacketListField(
                "Sids",
                [PLSAPR_TRANSLATED_SID_EX()],
                PLSAPR_TRANSLATED_SID_EX,
                size_is=lambda pkt: pkt.Entries,
            ),
            deferred=True,
        ),
    ]


class LsarLookupNames2_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("PolicyHandle", NDRContextHandle(), NDRContextHandle),
        NDRIntField("Count", None, size_of="Names"),
        NDRConfPacketListField(
            "Names",
            [PRPC_UNICODE_STRING()],
            PRPC_UNICODE_STRING,
            size_is=lambda pkt: pkt.Count,
        ),
        NDRPacketField(
            "TranslatedSids", PLSAPR_TRANSLATED_SIDS_EX(), PLSAPR_TRANSLATED_SIDS_EX
        ),
        NDRInt3264EnumField("LookupLevel", 0, LSAP_LOOKUP_LEVEL),
        NDRIntField("MappedCount", 0),
        NDRIntField("LookupOptions", 0),
        NDRIntField("ClientRevision", 0),
    ]


class LsarLookupNames2_Response(NDRPacket):
    fields_desc = [
        NDRFullPointerField(
            NDRPacketField(
                "ReferencedDomains",
                PLSAPR_REFERENCED_DOMAIN_LIST(),
                PLSAPR_REFERENCED_DOMAIN_LIST,
            )
        ),
        NDRPacketField(
            "TranslatedSids", PLSAPR_TRANSLATED_SIDS_EX(), PLSAPR_TRANSLATED_SIDS_EX
        ),
        NDRIntField("MappedCount", 0),
        NDRIntField("status", 0),
    ]


class PLSAPR_TRANSLATED_SID_EX2(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRInt3264EnumField("Use", 0, SID_NAME_USE),
        NDRFullPointerField(NDRPacketField("Sid", PRPC_SID(), PRPC_SID), deferred=True),
        NDRSignedIntField("DomainIndex", 0),
        NDRIntField("Flags", 0),
    ]


class PLSAPR_TRANSLATED_SIDS_EX2(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRIntField("Entries", None, size_of="Sids"),
        NDRFullPointerField(
            NDRConfPacketListField(
                "Sids",
                [PLSAPR_TRANSLATED_SID_EX2()],
                PLSAPR_TRANSLATED_SID_EX2,
                size_is=lambda pkt: pkt.Entries,
            ),
            deferred=True,
        ),
    ]


class LsarLookupNames3_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("PolicyHandle", NDRContextHandle(), NDRContextHandle),
        NDRIntField("Count", None, size_of="Names"),
        NDRConfPacketListField(
            "Names",
            [PRPC_UNICODE_STRING()],
            PRPC_UNICODE_STRING,
            size_is=lambda pkt: pkt.Count,
        ),
        NDRPacketField(
            "TranslatedSids", PLSAPR_TRANSLATED_SIDS_EX2(), PLSAPR_TRANSLATED_SIDS_EX2
        ),
        NDRInt3264EnumField("LookupLevel", 0, LSAP_LOOKUP_LEVEL),
        NDRIntField("MappedCount", 0),
        NDRIntField("LookupOptions", 0),
        NDRIntField("ClientRevision", 0),
    ]


class LsarLookupNames3_Response(NDRPacket):
    fields_desc = [
        NDRFullPointerField(
            NDRPacketField(
                "ReferencedDomains",
                PLSAPR_REFERENCED_DOMAIN_LIST(),
                PLSAPR_REFERENCED_DOMAIN_LIST,
            )
        ),
        NDRPacketField(
            "TranslatedSids", PLSAPR_TRANSLATED_SIDS_EX2(), PLSAPR_TRANSLATED_SIDS_EX2
        ),
        NDRIntField("MappedCount", 0),
        NDRIntField("status", 0),
    ]


class LsarLookupSids3_Request(NDRPacket):
    fields_desc = [
        NDRPacketField(
            "SidEnumBuffer", PLSAPR_SID_ENUM_BUFFER(), PLSAPR_SID_ENUM_BUFFER
        ),
        NDRPacketField(
            "TranslatedNames", PLSAPR_TRANSLATED_NAMES_EX(), PLSAPR_TRANSLATED_NAMES_EX
        ),
        NDRInt3264EnumField("LookupLevel", 0, LSAP_LOOKUP_LEVEL),
        NDRIntField("MappedCount", 0),
        NDRIntField("LookupOptions", 0),
        NDRIntField("ClientRevision", 0),
    ]


class LsarLookupSids3_Response(NDRPacket):
    fields_desc = [
        NDRFullPointerField(
            NDRPacketField(
                "ReferencedDomains",
                PLSAPR_REFERENCED_DOMAIN_LIST(),
                PLSAPR_REFERENCED_DOMAIN_LIST,
            )
        ),
        NDRPacketField(
            "TranslatedNames", PLSAPR_TRANSLATED_NAMES_EX(), PLSAPR_TRANSLATED_NAMES_EX
        ),
        NDRIntField("MappedCount", 0),
        NDRIntField("status", 0),
    ]


class LsarLookupNames4_Request(NDRPacket):
    fields_desc = [
        NDRIntField("Count", None, size_of="Names"),
        NDRConfPacketListField(
            "Names",
            [PRPC_UNICODE_STRING()],
            PRPC_UNICODE_STRING,
            size_is=lambda pkt: pkt.Count,
        ),
        NDRPacketField(
            "TranslatedSids", PLSAPR_TRANSLATED_SIDS_EX2(), PLSAPR_TRANSLATED_SIDS_EX2
        ),
        NDRInt3264EnumField("LookupLevel", 0, LSAP_LOOKUP_LEVEL),
        NDRIntField("MappedCount", 0),
        NDRIntField("LookupOptions", 0),
        NDRIntField("ClientRevision", 0),
    ]


class LsarLookupNames4_Response(NDRPacket):
    fields_desc = [
        NDRFullPointerField(
            NDRPacketField(
                "ReferencedDomains",
                PLSAPR_REFERENCED_DOMAIN_LIST(),
                PLSAPR_REFERENCED_DOMAIN_LIST,
            )
        ),
        NDRPacketField(
            "TranslatedSids", PLSAPR_TRANSLATED_SIDS_EX2(), PLSAPR_TRANSLATED_SIDS_EX2
        ),
        NDRIntField("MappedCount", 0),
        NDRIntField("status", 0),
    ]


LSARPC_OPNUMS = {
    0: DceRpcOp(LsarClose_Request, LsarClose_Response),
    # 1: Opnum1NotUsedOnWire,
    # 2: Lsar_LSA_DP_2,
    # 3: Lsar_LSA_DP_3,
    # 4: Lsar_LSA_DP_4,
    # 5: Opnum5NotUsedOnWire,
    6: DceRpcOp(LsarOpenPolicy_Request, LsarOpenPolicy_Response),
    # 7: Lsar_LSA_DP_7,
    # 8: Lsar_LSA_DP_8,
    # 9: Opnum9NotUsedOnWire,
    # 10: Lsar_LSA_DP_10,
    # 11: Lsar_LSA_DP_11,
    # 12: Lsar_LSA_DP_12,
    # 13: Lsar_LSA_DP_13,
    14: DceRpcOp(LsarLookupNames_Request, LsarLookupNames_Response),
    15: DceRpcOp(LsarLookupSids_Request, LsarLookupSids_Response),
    # 16: Lsar_LSA_DP_16,
    # 17: Lsar_LSA_DP_17,
    # 18: Lsar_LSA_DP_18,
    # 19: Lsar_LSA_DP_19,
    # 20: Lsar_LSA_DP_20,
    # 21: Opnum21NotUsedOnWire,
    # 22: Opnum22NotUsedOnWire,
    # 23: Lsar_LSA_DP_23,
    # 24: Lsar_LSA_DP_24,
    # 25: Lsar_LSA_DP_25,
    # 26: Lsar_LSA_DP_26,
    # 27: Lsar_LSA_DP_27,
    # 28: Lsar_LSA_DP_28,
    # 29: Lsar_LSA_DP_29,
    # 30: Lsar_LSA_DP_30,
    # 31: Lsar_LSA_DP_31,
    # 32: Lsar_LSA_DP_32,
    # 33: Lsar_LSA_DP_33,
    # 34: Lsar_LSA_DP_34,
    # 35: Lsar_LSA_DP_35,
    # 36: Lsar_LSA_DP_36,
    # 37: Lsar_LSA_DP_37,
    # 38: Lsar_LSA_DP_38,
    # 39: Lsar_LSA_DP_39,
    # 40: Lsar_LSA_DP_40,
    # 41: Lsar_LSA_DP_41,
    # 42: Lsar_LSA_DP_42,
    # 43: Lsar_LSA_DP_43,
    44: DceRpcOp(LsarOpenPolicy2_Request, LsarOpenPolicy2_Response),
    45: DceRpcOp(LsarGetUserName_Request, LsarGetUserName_Response),
    # 46: Lsar_LSA_DP_46,
    # 47: Lsar_LSA_DP_47,
    # 48: Lsar_LSA_DP_48,
    # 49: Lsar_LSA_DP_49,
    # 50: Lsar_LSA_DP_50,
    # 51: Lsar_LSA_DP_51,
    # 52: Opnum52NotUsedOnWire,
    # 53: Lsar_LSA_DP_53,
    # 54: Lsar_LSA_DP_54,
    # 55: Lsar_LSA_DP_55,
    # 56: Opnum56NotUsedOnWire,
    57: DceRpcOp(LsarLookupSids2_Request, LsarLookupSids2_Response),
    58: DceRpcOp(LsarLookupNames2_Request, LsarLookupNames2_Response),
    # 59: Lsar_LSA_DP_59,
    # 60: Opnum60NotUsedOnWire,
    # 61: Opnum61NotUsedOnWire,
    # 62: Opnum62NotUsedOnWire,
    # 63: Opnum63NotUsedOnWire,
    # 64: Opnum64NotUsedOnWire,
    # 65: Opnum65NotUsedOnWire,
    # 66: Opnum66NotUsedOnWire,
    # 67: Opnum67NotUsedOnWire,
    68: DceRpcOp(LsarLookupNames3_Request, LsarLookupNames3_Response),
    # 69: Opnum69NotUsedOnWire,
    # 70: Opnum70NotUsedOnWire,
    # 71: Opnum71NotUsedOnWire,
    # 72: Opnum72NotUsedOnWire,
    # 73: Lsar_LSA_DP_73,
    # 74: Lsar_LSA_DP_74,
    # 75: Opnum75NotUsedOnWire,
    76: DceRpcOp(LsarLookupSids3_Request, LsarLookupSids3_Response),
    77: DceRpcOp(LsarLookupNames4_Request, LsarLookupNames4_Response),
}
register_dcerpc_interface(
    name="lsarpc",
    uuid=uuid.UUID("12345778-1234-ABCD-EF00-0123456789AB"),
    version="0.0",
    opnums=LSARPC_OPNUMS,
)



### ms-wsrm.idl
"""
RPC definitions for the following interfaces:
- IResourceManager (v0.0): C5CEBEE2-9DF5-4CDD-A08C-C2471BC144B4
- IWRMCalendar (v0.0): 481E06CF-AB04-4498-8FFE-124A0A34296D
- IWRMPolicy (v0.0): 59602EB6-57B0-4FD8-AA4B-EBF06971FE15
- IWRMResourceGroup (v0.0): BC681469-9DD9-4BF4-9B3D-709F69EFE431
- IWRMAccounting (v0.0): 4F7CA01C-A9E5-45B6-B142-2332A1339C1D
- IWRMConfig (v0.0): 21546AE8-4DA5-445E-987F-627FEA39C5E8
- IWRMProtocol (v0.0): F31931A9-832D-481C-9503-887A0E6A79F0
- IWRMMachineGroup (v0.0): 943991a5-b3fe-41fa-9696-7f7b656ee34b
- IResourceManager2 (v0.0): 2A3EB639-D134-422d-90D8-AAA1B5216202
- IWRMRemoteSessionMgmt (v0.0): FC910418-55CA-45ef-B264-83D4CE7D30E0
This file is auto-generated by midl-to-scapy, do not modify.
"""

from enum import IntEnum
import uuid


from scapy.layers.dcerpc import (
    NDRPacket,
    DceRpcOp,
    NDRConfStrLenField,
    NDRConfStrLenFieldUtf16,
    NDRFullPointerField,
    NDRIntEnumField,
    NDRIntField,
    NDRLongField,
    NDRPacketField,
    NDRSignedIntField,
    register_com_interface,
)


class FLAGGED_WORD_BLOB(NDRPacket):
    ALIGNMENT = (4, 8)
    DEPORTED_CONFORMANTS = ["asData"]
    fields_desc = [
        NDRIntField("cBytes", 0),
        NDRIntField("clSize", None, size_of="asData"),
        NDRConfStrLenFieldUtf16(
            "asData", "", size_is=lambda pkt: pkt.clSize, conformant_in_struct=True
        ),
    ]


class RetrieveEventList_Request(NDRPacket):
    fields_desc = []


class RetrieveEventList_Response(NDRPacket):
    fields_desc = [
        NDRFullPointerField(
            NDRPacketField("pbstrEventList", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB)
        ),
        NDRIntField("status", 0),
    ]


class GetSystemAffinity_Request(NDRPacket):
    fields_desc = []


class GetSystemAffinity_Response(NDRPacket):
    fields_desc = [NDRLongField("pdwSysAffinity", 0), NDRIntField("status", 0)]


class ImportXMLFiles_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("bstrPMCXml", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB),
        NDRPacketField("bstrPolicyXml", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB),
        NDRPacketField("bstrCalendarXml", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB),
        NDRPacketField("bstrConditionalXml", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB),
    ]


class ImportXMLFiles_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class ExportXMLFiles_Request(NDRPacket):
    fields_desc = []


class ExportXMLFiles_Response(NDRPacket):
    fields_desc = [
        NDRFullPointerField(
            NDRPacketField("pbstrPMCXml", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB)
        ),
        NDRFullPointerField(
            NDRPacketField("pbstrPolicyXml", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB)
        ),
        NDRFullPointerField(
            NDRPacketField("pbstrCalendarXml", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB)
        ),
        NDRFullPointerField(
            NDRPacketField(
                "pbstrConditionalXml", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB
            )
        ),
        NDRIntField("status", 0),
    ]


class RESTORE_MODE(IntEnum):
    RESTORE_LAST_GOOD_STATE = 1
    RESTORE_EMPTY_FILES = 2


class RestoreXMLFiles_Request(NDRPacket):
    fields_desc = [NDRIntEnumField("enumRestore", 0, RESTORE_MODE)]


class RestoreXMLFiles_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class OBJECT_TYPE(IntEnum):
    OBJECT_SELECTION_CRITERIA = 1
    OBJECT_POLICY = 2
    OBJECT_SCHEDULE = 3


class GetDependencies_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("bstrObjectName", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB),
        NDRIntEnumField("enumObject", 0, OBJECT_TYPE),
    ]


class GetDependencies_Response(NDRPacket):
    fields_desc = [
        NDRFullPointerField(
            NDRPacketField(
                "pbstrDependencyList", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB
            )
        ),
        NDRIntField("status", 0),
    ]


class GetServiceList_Request(NDRPacket):
    fields_desc = []


class GetServiceList_Response(NDRPacket):
    fields_desc = [
        NDRFullPointerField(
            NDRPacketField("pbstrServiceList", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB)
        ),
        NDRIntField("status", 0),
    ]


class GetIISAppPoolNames_Request(NDRPacket):
    fields_desc = []


class GetIISAppPoolNames_Response(NDRPacket):
    fields_desc = [
        NDRFullPointerField(
            NDRPacketField(
                "pbstrIISAppPoolList", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB
            )
        ),
        NDRFullPointerField(
            NDRPacketField(
                "pbstrSystemDirectory", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB
            )
        ),
        NDRIntField("status", 0),
    ]


class GetServerName_Request(NDRPacket):
    fields_desc = []


class GetServerName_Response(NDRPacket):
    fields_desc = [
        NDRFullPointerField(
            NDRPacketField("pbstrServerName", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB)
        ),
        NDRIntField("status", 0),
    ]


class GetCurrentMemory_Request(NDRPacket):
    fields_desc = []


class GetCurrentMemory_Response(NDRPacket):
    fields_desc = [NDRLongField("pdwCurrMemory", 0), NDRIntField("status", 0)]


IRESOURCEMANAGER_OPNUMS = {
    0: DceRpcOp(RetrieveEventList_Request, RetrieveEventList_Response),
    1: DceRpcOp(GetSystemAffinity_Request, GetSystemAffinity_Response),
    2: DceRpcOp(ImportXMLFiles_Request, ImportXMLFiles_Response),
    3: DceRpcOp(ExportXMLFiles_Request, ExportXMLFiles_Response),
    4: DceRpcOp(RestoreXMLFiles_Request, RestoreXMLFiles_Response),
    5: DceRpcOp(GetDependencies_Request, GetDependencies_Response),
    6: DceRpcOp(GetServiceList_Request, GetServiceList_Response),
    7: DceRpcOp(GetIISAppPoolNames_Request, GetIISAppPoolNames_Response),
    8: DceRpcOp(GetServerName_Request, GetServerName_Response),
    9: DceRpcOp(GetCurrentMemory_Request, GetCurrentMemory_Response),
}
register_com_interface(
    name="IResourceManager",
    uuid=uuid.UUID("C5CEBEE2-9DF5-4CDD-A08C-C2471BC144B4"),
    opnums=IRESOURCEMANAGER_OPNUMS,
)


class GetCalendarInfo_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("bstrCalendarName", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB)
    ]


class GetCalendarInfo_Response(NDRPacket):
    fields_desc = [
        NDRFullPointerField(
            NDRPacketField("pbstrCalendarXML", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB)
        ),
        NDRIntField("status", 0),
    ]


class CreateCalendar_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("bstrCalendarXML", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB),
        NDRSignedIntField("bChangeActivePolicy", 0),
    ]


class CreateCalendar_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class ModifyCalendar_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("bstrCalendarXML", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB),
        NDRSignedIntField("bOverwrite", 0),
        NDRSignedIntField("bChangeActivePolicy", 0),
    ]


class ModifyCalendar_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class DeleteCalendar_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("bstrCalendarName", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB),
        NDRSignedIntField("bChangeActivePolicy", 0),
    ]


class DeleteCalendar_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class RenameCalendar_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("bstrOldCalendarName", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB),
        NDRPacketField("bstrNewCalendarName", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB),
    ]


class RenameCalendar_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class ComputeEvents_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("szStartTime", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB),
        NDRPacketField("szEndTime", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB),
        NDRSignedIntField("fMergeEvents", 0),
    ]


class ComputeEvents_Response(NDRPacket):
    fields_desc = [
        NDRFullPointerField(
            NDRPacketField("pbstrEvents", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB)
        ),
        NDRFullPointerField(
            NDRPacketField("pbstrConflicts", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB)
        ),
        NDRIntField("status", 0),
    ]


class GetScheduleInfo_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("bstrScheduleName", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB)
    ]


class GetScheduleInfo_Response(NDRPacket):
    fields_desc = [
        NDRFullPointerField(
            NDRPacketField("pbstrScheduleXML", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB)
        ),
        NDRIntField("status", 0),
    ]


class CreateSchedule_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("bstrScheduleXML", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB)
    ]


class CreateSchedule_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class ModifySchedule_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("bstrScheduleXML", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB),
        NDRSignedIntField("bOverwrite", 0),
        NDRSignedIntField("bChangeActivePolicy", 0),
    ]


class ModifySchedule_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class DeleteSchedule_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("bstrScheduleName", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB)
    ]


class DeleteSchedule_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class RenameSchedule_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("bstrOldScheduleName", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB),
        NDRPacketField("bstrNewScheduleName", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB),
    ]


class RenameSchedule_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class MoveBeforeCalendar_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("bstrCalendarName", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB),
        NDRPacketField("bstrRefCalendarName", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB),
        NDRSignedIntField("bChangeActivePolicy", 0),
    ]


class MoveBeforeCalendar_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class MoveAfterCalendar_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("bstrCalendarName", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB),
        NDRPacketField("bstrRefCalendarName", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB),
        NDRSignedIntField("bChangeActivePolicy", 0),
    ]


class MoveAfterCalendar_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class GetServerTimeZone_Request(NDRPacket):
    fields_desc = []


class GetServerTimeZone_Response(NDRPacket):
    fields_desc = [NDRSignedIntField("pnServerTimeZone", 0), NDRIntField("status", 0)]


IWRMCALENDAR_OPNUMS = {
    0: DceRpcOp(GetCalendarInfo_Request, GetCalendarInfo_Response),
    1: DceRpcOp(CreateCalendar_Request, CreateCalendar_Response),
    2: DceRpcOp(ModifyCalendar_Request, ModifyCalendar_Response),
    3: DceRpcOp(DeleteCalendar_Request, DeleteCalendar_Response),
    4: DceRpcOp(RenameCalendar_Request, RenameCalendar_Response),
    5: DceRpcOp(ComputeEvents_Request, ComputeEvents_Response),
    6: DceRpcOp(GetScheduleInfo_Request, GetScheduleInfo_Response),
    7: DceRpcOp(CreateSchedule_Request, CreateSchedule_Response),
    8: DceRpcOp(ModifySchedule_Request, ModifySchedule_Response),
    9: DceRpcOp(DeleteSchedule_Request, DeleteSchedule_Response),
    10: DceRpcOp(RenameSchedule_Request, RenameSchedule_Response),
    11: DceRpcOp(MoveBeforeCalendar_Request, MoveBeforeCalendar_Response),
    12: DceRpcOp(MoveAfterCalendar_Request, MoveAfterCalendar_Response),
    13: DceRpcOp(GetServerTimeZone_Request, GetServerTimeZone_Response),
}
register_com_interface(
    name="IWRMCalendar",
    uuid=uuid.UUID("481E06CF-AB04-4498-8FFE-124A0A34296D"),
    opnums=IWRMCALENDAR_OPNUMS,
)


class GetPolicyInfo_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("bstrPolicyName", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB)
    ]


class GetPolicyInfo_Response(NDRPacket):
    fields_desc = [
        NDRFullPointerField(
            NDRPacketField("pbstrPolicyInfo", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB)
        ),
        NDRIntField("status", 0),
    ]


class CreatePolicy_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("bstrPolicyInfo", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB)
    ]


class CreatePolicy_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class ModifyPolicy_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("bstrPolicyInfo", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB),
        NDRSignedIntField("bOverwrite", 0),
    ]


class ModifyPolicy_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class DeletePolicy_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("bstrPolicyName", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB)
    ]


class DeletePolicy_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class RenameAllocationPolicy_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("bstrNewPolicyName", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB),
        NDRPacketField("bstrOldPolicyName", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB),
    ]


class RenameAllocationPolicy_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class MoveBefore_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("bstrPolicyName", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB),
        NDRPacketField("bstrResourceGroupName", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB),
        NDRPacketField(
            "bstrRefResourceGroupName", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB
        ),
    ]


class MoveBefore_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class MoveAfter_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("bstrPolicyName", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB),
        NDRPacketField("bstrResourceGroupName", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB),
        NDRPacketField(
            "bstrRefResourceGroupName", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB
        ),
    ]


class MoveAfter_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class SetCalDefaultPolicyName_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("bstrDefaultPolicyName", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB)
    ]


class SetCalDefaultPolicyName_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class GetCalDefaultPolicyName_Request(NDRPacket):
    fields_desc = []


class GetCalDefaultPolicyName_Response(NDRPacket):
    fields_desc = [
        NDRFullPointerField(
            NDRPacketField(
                "pbstrDefaultPolicyName", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB
            )
        ),
        NDRIntField("status", 0),
    ]


class GetProcessList_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("bstrPolicyName", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB)
    ]


class GetProcessList_Response(NDRPacket):
    fields_desc = [
        NDRFullPointerField(
            NDRPacketField("pbstrProcessList", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB)
        ),
        NDRIntField("status", 0),
    ]


class MANAGEMENT_TYPE(IntEnum):
    MANUAL_ACTIVE_POLICY = 1
    CALENDAR_POLICY = 2
    PROFILING = 3


class GetCurrentPolicy_Request(NDRPacket):
    fields_desc = []


class GetCurrentPolicy_Response(NDRPacket):
    fields_desc = [
        NDRFullPointerField(
            NDRPacketField(
                "pbstrCurrentPolicyInfo", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB
            )
        ),
        NDRIntEnumField("enumManage", 0, MANAGEMENT_TYPE),
        NDRIntField("status", 0),
    ]


class SetCurrentPolicy_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("bstrPolicyName", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB),
        NDRIntEnumField("enumManage", 0, MANAGEMENT_TYPE),
    ]


class SetCurrentPolicy_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class GetCurrentStateAndActivePolicyName_Request(NDRPacket):
    fields_desc = []


class GetCurrentStateAndActivePolicyName_Response(NDRPacket):
    fields_desc = [
        NDRFullPointerField(
            NDRPacketField(
                "pbstrCurrentPolicyName", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB
            )
        ),
        NDRIntEnumField("enumManage", 0, MANAGEMENT_TYPE),
        NDRIntField("status", 0),
    ]


class GetConditionalPolicy_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("bstrPolicyName", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB)
    ]


class GetConditionalPolicy_Response(NDRPacket):
    fields_desc = [
        NDRFullPointerField(
            NDRPacketField("pbstrPolicyInfo", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB)
        ),
        NDRIntField("status", 0),
    ]


class SetConditionalPolicy_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("bstrPolicyInfo", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB)
    ]


class SetConditionalPolicy_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


IWRMPOLICY_OPNUMS = {
    0: DceRpcOp(GetPolicyInfo_Request, GetPolicyInfo_Response),
    1: DceRpcOp(CreatePolicy_Request, CreatePolicy_Response),
    2: DceRpcOp(ModifyPolicy_Request, ModifyPolicy_Response),
    3: DceRpcOp(DeletePolicy_Request, DeletePolicy_Response),
    4: DceRpcOp(RenameAllocationPolicy_Request, RenameAllocationPolicy_Response),
    5: DceRpcOp(MoveBefore_Request, MoveBefore_Response),
    6: DceRpcOp(MoveAfter_Request, MoveAfter_Response),
    7: DceRpcOp(SetCalDefaultPolicyName_Request, SetCalDefaultPolicyName_Response),
    8: DceRpcOp(GetCalDefaultPolicyName_Request, GetCalDefaultPolicyName_Response),
    9: DceRpcOp(GetProcessList_Request, GetProcessList_Response),
    10: DceRpcOp(GetCurrentPolicy_Request, GetCurrentPolicy_Response),
    11: DceRpcOp(SetCurrentPolicy_Request, SetCurrentPolicy_Response),
    12: DceRpcOp(
        GetCurrentStateAndActivePolicyName_Request,
        GetCurrentStateAndActivePolicyName_Response,
    ),
    13: DceRpcOp(GetConditionalPolicy_Request, GetConditionalPolicy_Response),
    14: DceRpcOp(SetConditionalPolicy_Request, SetConditionalPolicy_Response),
}
register_com_interface(
    name="IWRMPolicy",
    uuid=uuid.UUID("59602EB6-57B0-4FD8-AA4B-EBF06971FE15"),
    opnums=IWRMPOLICY_OPNUMS,
)


class GetResourceGroupInfo_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("bstrResourceGroupName", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB)
    ]


class GetResourceGroupInfo_Response(NDRPacket):
    fields_desc = [
        NDRFullPointerField(
            NDRPacketField(
                "pbstrResourceGroupInfo", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB
            )
        ),
        NDRIntField("status", 0),
    ]


class ModifyResourceGroup_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("bstrResourceGroupInfo", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB),
        NDRSignedIntField("bOverwrite", 0),
    ]


class ModifyResourceGroup_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class CreateResourceGroup_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("bstrResourceGroupInfo", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB)
    ]


class CreateResourceGroup_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class DeleteResourceGroup_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("bstrResourceGroupName", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB)
    ]


class DeleteResourceGroup_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class RenameResourceGroup_Request(NDRPacket):
    fields_desc = [
        NDRPacketField(
            "bstrNewResourceGroupName", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB
        ),
        NDRPacketField(
            "bstrOldResourceGroupName", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB
        ),
    ]


class RenameResourceGroup_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


IWRMRESOURCEGROUP_OPNUMS = {
    0: DceRpcOp(GetResourceGroupInfo_Request, GetResourceGroupInfo_Response),
    1: DceRpcOp(ModifyResourceGroup_Request, ModifyResourceGroup_Response),
    2: DceRpcOp(CreateResourceGroup_Request, CreateResourceGroup_Response),
    3: DceRpcOp(DeleteResourceGroup_Request, DeleteResourceGroup_Response),
    4: DceRpcOp(RenameResourceGroup_Request, RenameResourceGroup_Response),
}
register_com_interface(
    name="IWRMResourceGroup",
    uuid=uuid.UUID("BC681469-9DD9-4BF4-9B3D-709F69EFE431"),
    opnums=IWRMRESOURCEGROUP_OPNUMS,
)


class CreateAccountingDb_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("bstrServerName", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB),
        NDRSignedIntField("bWindowsAuth", 0),
        NDRPacketField("bstrUserName", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB),
        NDRPacketField("bstrPasswd", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB),
    ]


class CreateAccountingDb_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class GetAccountingMetadata_Request(NDRPacket):
    fields_desc = []


class GetAccountingMetadata_Response(NDRPacket):
    fields_desc = [
        NDRFullPointerField(
            NDRPacketField("pbstrMetaData", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB)
        ),
        NDRIntField("status", 0),
    ]


class ExecuteAccountingQuery_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("bstrAccountingQuery", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB),
        NDRPacketField("bstrStartingDate", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB),
        NDRPacketField("bstrEndingDate", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB),
    ]


class ExecuteAccountingQuery_Response(NDRPacket):
    fields_desc = [
        NDRFullPointerField(
            NDRPacketField("pbstrResult", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB)
        ),
        NDRSignedIntField("pbIsThereMoreData", 0),
        NDRIntField("status", 0),
    ]


class GetRawAccountingData_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("bstrStartingDate", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB),
        NDRPacketField("bstrEndingDate", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB),
        NDRPacketField("bstrMachineName", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB),
    ]


class GetRawAccountingData_Response(NDRPacket):
    fields_desc = [
        NDRFullPointerField(
            NDRPacketField("pbstrResult", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB)
        ),
        NDRSignedIntField("pbIsThereMoreData", 0),
        NDRIntField("status", 0),
    ]


class GetNextAccountingDataBatch_Request(NDRPacket):
    fields_desc = []


class GetNextAccountingDataBatch_Response(NDRPacket):
    fields_desc = [
        NDRFullPointerField(
            NDRPacketField("pbstrResult", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB)
        ),
        NDRSignedIntField("pbIsThereMoreData", 0),
        NDRIntField("status", 0),
    ]


class DeleteAccountingData_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("bstrStartingDate", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB),
        NDRPacketField("bstrEndingDate", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB),
        NDRPacketField("bstrMachineName", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB),
    ]


class DeleteAccountingData_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class DefragmentDB_Request(NDRPacket):
    fields_desc = []


class DefragmentDB_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class CancelAccountingQuery_Request(NDRPacket):
    fields_desc = [NDRSignedIntField("flag", 0)]


class CancelAccountingQuery_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class RegisterAccountingClient_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("bstrClientId", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB)
    ]


class RegisterAccountingClient_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class DumpAccountingData_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("bstrAccountingData", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB)
    ]


class DumpAccountingData_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class GetAccountingClients_Request(NDRPacket):
    fields_desc = []


class GetAccountingClients_Response(NDRPacket):
    fields_desc = [
        NDRFullPointerField(
            NDRPacketField("pbstrClientIds", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB)
        ),
        NDRIntField("status", 0),
    ]


class SetAccountingClientStatus_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("bstrClientIds", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB)
    ]


class SetAccountingClientStatus_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class CheckAccountingConnection_Request(NDRPacket):
    fields_desc = []


class CheckAccountingConnection_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class SetClientPermissions_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("bstrClientId", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB),
        NDRSignedIntField("fAddPermissions", 0),
    ]


class SetClientPermissions_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


IWRMACCOUNTING_OPNUMS = {
    0: DceRpcOp(CreateAccountingDb_Request, CreateAccountingDb_Response),
    1: DceRpcOp(GetAccountingMetadata_Request, GetAccountingMetadata_Response),
    2: DceRpcOp(ExecuteAccountingQuery_Request, ExecuteAccountingQuery_Response),
    3: DceRpcOp(GetRawAccountingData_Request, GetRawAccountingData_Response),
    4: DceRpcOp(
        GetNextAccountingDataBatch_Request, GetNextAccountingDataBatch_Response
    ),
    5: DceRpcOp(DeleteAccountingData_Request, DeleteAccountingData_Response),
    6: DceRpcOp(DefragmentDB_Request, DefragmentDB_Response),
    7: DceRpcOp(CancelAccountingQuery_Request, CancelAccountingQuery_Response),
    8: DceRpcOp(RegisterAccountingClient_Request, RegisterAccountingClient_Response),
    9: DceRpcOp(DumpAccountingData_Request, DumpAccountingData_Response),
    10: DceRpcOp(GetAccountingClients_Request, GetAccountingClients_Response),
    11: DceRpcOp(SetAccountingClientStatus_Request, SetAccountingClientStatus_Response),
    12: DceRpcOp(CheckAccountingConnection_Request, CheckAccountingConnection_Response),
    13: DceRpcOp(SetClientPermissions_Request, SetClientPermissions_Response),
}
register_com_interface(
    name="IWRMAccounting",
    uuid=uuid.UUID("4F7CA01C-A9E5-45B6-B142-2332A1339C1D"),
    opnums=IWRMACCOUNTING_OPNUMS,
)


class CONFIGTYPE(IntEnum):
    CONFIGTYPE_ACCOUNTING = 1
    CONFIGTYPE_NOTIFICATION = 2
    CONFIGTYPE_CALENDARING = 3


class GetConfig_Request(NDRPacket):
    fields_desc = [NDRIntEnumField("enumConfigType", 0, CONFIGTYPE)]


class GetConfig_Response(NDRPacket):
    fields_desc = [
        NDRFullPointerField(
            NDRPacketField("pbstrConfigInfo", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB)
        ),
        NDRIntField("status", 0),
    ]


class SetConfig_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("bstrConfigInfo", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB),
        NDRIntEnumField("enumConfigType", 0, CONFIGTYPE),
    ]


class SetConfig_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class IsEnabled_Request(NDRPacket):
    fields_desc = [NDRIntEnumField("enumConfigType", 0, CONFIGTYPE)]


class IsEnabled_Response(NDRPacket):
    fields_desc = [NDRSignedIntField("pbEnable", 0), NDRIntField("status", 0)]


class EnableDisable_Request(NDRPacket):
    fields_desc = [
        NDRSignedIntField("bEnableDisable", 0),
        NDRIntEnumField("enumConfigType", 0, CONFIGTYPE),
    ]


class EnableDisable_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class EXCLUSIONLIST_TYPE(IntEnum):
    SYSTEM_EXCLUSION_LIST = 1
    USER_EXCLUSION_LIST = 2
    DEFAULT_USER_EXCLUSION_LIST = 4


class GetExclusionList_Request(NDRPacket):
    fields_desc = [NDRIntEnumField("enumListType", 0, EXCLUSIONLIST_TYPE)]


class GetExclusionList_Response(NDRPacket):
    fields_desc = [
        NDRFullPointerField(
            NDRPacketField("pbstrExclusionList", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB)
        ),
        NDRIntField("status", 0),
    ]


class SetExclusionList_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("bstrExclusionList", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB)
    ]


class SetExclusionList_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class WSRMActivate_Request(NDRPacket):
    fields_desc = [NDRSignedIntField("bActivate", 0)]


class WSRMActivate_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class IsWSRMActivated_Request(NDRPacket):
    fields_desc = []


class IsWSRMActivated_Response(NDRPacket):
    fields_desc = [NDRSignedIntField("pbActivated", 0), NDRIntField("status", 0)]


class RestoreExclusionList_Request(NDRPacket):
    fields_desc = []


class RestoreExclusionList_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


IWRMCONFIG_OPNUMS = {
    0: DceRpcOp(GetConfig_Request, GetConfig_Response),
    1: DceRpcOp(SetConfig_Request, SetConfig_Response),
    2: DceRpcOp(IsEnabled_Request, IsEnabled_Response),
    3: DceRpcOp(EnableDisable_Request, EnableDisable_Response),
    4: DceRpcOp(GetExclusionList_Request, GetExclusionList_Response),
    5: DceRpcOp(SetExclusionList_Request, SetExclusionList_Response),
    6: DceRpcOp(WSRMActivate_Request, WSRMActivate_Response),
    7: DceRpcOp(IsWSRMActivated_Request, IsWSRMActivated_Response),
    8: DceRpcOp(RestoreExclusionList_Request, RestoreExclusionList_Response),
}
register_com_interface(
    name="IWRMConfig",
    uuid=uuid.UUID("21546AE8-4DA5-445E-987F-627FEA39C5E8"),
    opnums=IWRMCONFIG_OPNUMS,
)


class GetSupportedClient_Request(NDRPacket):
    fields_desc = []


class GetSupportedClient_Response(NDRPacket):
    fields_desc = [
        NDRFullPointerField(
            NDRPacketField(
                "pbstrSupportedClients", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB
            )
        ),
        NDRIntField("status", 0),
    ]


IWRMPROTOCOL_OPNUMS = {
    0: DceRpcOp(GetSupportedClient_Request, GetSupportedClient_Response)
}
register_com_interface(
    name="IWRMProtocol",
    uuid=uuid.UUID("F31931A9-832D-481C-9503-887A0E6A79F0"),
    opnums=IWRMPROTOCOL_OPNUMS,
)


class CreateMachineGroup_Request(NDRPacket):
    fields_desc = [
        NDRPacketField(
            "bstrParentMachineGroupId", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB
        ),
        NDRPacketField("bstrMachineGroupInfo", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB),
    ]


class CreateMachineGroup_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class GetMachineGroupInfo_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("bstrMachineGroupId", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB)
    ]


class GetMachineGroupInfo_Response(NDRPacket):
    fields_desc = [
        NDRFullPointerField(
            NDRPacketField(
                "pbstrMachineGroupInfo", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB
            )
        ),
        NDRIntField("status", 0),
    ]


class MACHINE_GROUP_MERGE_OPTIONS(IntEnum):
    OVERWRITE_MG_MERGE_OPTION = 1
    OVERRIDE_MG_MERGE_OPTION = 2
    APPEND_MG_MERGE_OPTION = 3
    SMART_MG_MERGE_OPTION = 4


class ModifyMachineGroup_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("bstrMachineGroupId", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB),
        NDRPacketField("bstrMachineGroupInfo", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB),
        NDRIntEnumField("enumMGMergeOptions", 0, MACHINE_GROUP_MERGE_OPTIONS),
    ]


class ModifyMachineGroup_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class DeleteMachineGroup_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("bstrMachineGroupId", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB)
    ]


class DeleteMachineGroup_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class RenameMachineGroup_Request(NDRPacket):
    fields_desc = [
        NDRPacketField(
            "bstrOldMachineGroupName", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB
        ),
        NDRPacketField(
            "bstrNewMachineGroupName", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB
        ),
    ]


class RenameMachineGroup_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class AddMachine_Request(NDRPacket):
    fields_desc = [
        NDRPacketField(
            "bstrParentMachineGroupId", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB
        ),
        NDRPacketField("bstrMachineInfo", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB),
    ]


class AddMachine_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class GetMachineInfo_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("bstrMachineId", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB)
    ]


class GetMachineInfo_Response(NDRPacket):
    fields_desc = [
        NDRFullPointerField(
            NDRPacketField("pbstrMachineInfo", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB)
        ),
        NDRIntField("status", 0),
    ]


class ModifyMachineInfo_Request(NDRPacket):
    fields_desc = [
        NDRPacketField(
            "bstrParentMachineGroupId", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB
        ),
        NDRPacketField("bstrMachineId", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB),
        NDRPacketField("bstrMachineInfo", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB),
    ]


class ModifyMachineInfo_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class DeleteMachine_Request(NDRPacket):
    fields_desc = [
        NDRPacketField(
            "bstrParentMachineGroupId", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB
        ),
        NDRPacketField("bstrMachineId", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB),
    ]


class DeleteMachine_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


IWRMMACHINEGROUP_OPNUMS = {
    0: DceRpcOp(CreateMachineGroup_Request, CreateMachineGroup_Response),
    1: DceRpcOp(GetMachineGroupInfo_Request, GetMachineGroupInfo_Response),
    2: DceRpcOp(ModifyMachineGroup_Request, ModifyMachineGroup_Response),
    3: DceRpcOp(DeleteMachineGroup_Request, DeleteMachineGroup_Response),
    4: DceRpcOp(RenameMachineGroup_Request, RenameMachineGroup_Response),
    5: DceRpcOp(AddMachine_Request, AddMachine_Response),
    6: DceRpcOp(GetMachineInfo_Request, GetMachineInfo_Response),
    7: DceRpcOp(ModifyMachineInfo_Request, ModifyMachineInfo_Response),
    8: DceRpcOp(DeleteMachine_Request, DeleteMachine_Response),
}
register_com_interface(
    name="IWRMMachineGroup",
    uuid=uuid.UUID("943991a5-b3fe-41fa-9696-7f7b656ee34b"),
    opnums=IWRMMACHINEGROUP_OPNUMS,
)


class ExportObjects_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("bstrObjectIds", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB),
        NDRIntEnumField("enumObjectType", 0, OBJECT_TYPE),
    ]


class ExportObjects_Response(NDRPacket):
    fields_desc = [
        NDRFullPointerField(
            NDRPacketField("pbstrObjectXml", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB)
        ),
        NDRIntField("status", 0),
    ]


class GetImportConflicts_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("bstrPMCXml", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB),
        NDRPacketField("bstrPolicyXml", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB),
        NDRPacketField("bstrCalendarXml", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB),
        NDRPacketField("bstrConditionalXml", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB),
        NDRPacketField("bstrMachineGroupXml", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB),
        NDRPacketField("bstrConfigurationXmls", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB),
    ]


class GetImportConflicts_Response(NDRPacket):
    fields_desc = [
        NDRFullPointerField(
            NDRPacketField(
                "pbstrConflictingObjects", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB
            )
        ),
        NDRIntField("status", 0),
    ]


class IMPORT_TYPE(IntEnum):
    OVERWRITE_IMPORT = 1
    IGNORE_EXISTING_IMPORT = 2
    OVERRIDE_EXISTING_IMPORT = 3
    SMART_MERGE_RENAME_EXISTING_IMPORT = 4
    SMART_MERGE_RENAME_IMPORTED_IMPORT = 5


class ImportXml_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("bstrPMCXml", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB),
        NDRPacketField("bstrPolicyXml", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB),
        NDRPacketField("bstrCalendarXml", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB),
        NDRPacketField("bstrConditionalXml", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB),
        NDRPacketField("bstrMachineGroupXml", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB),
        NDRPacketField("bstrConfigurationXmls", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB),
        NDRIntEnumField("enumImportType", 0, IMPORT_TYPE),
    ]


class ImportXml_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class ExportXml_Request(NDRPacket):
    fields_desc = []


class ExportXml_Response(NDRPacket):
    fields_desc = [
        NDRFullPointerField(
            NDRPacketField("pbstrPMCXml", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB)
        ),
        NDRFullPointerField(
            NDRPacketField("pbstrPolicyXml", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB)
        ),
        NDRFullPointerField(
            NDRPacketField("pbstrCalendarXml", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB)
        ),
        NDRFullPointerField(
            NDRPacketField(
                "pbstrConditionalXml", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB
            )
        ),
        NDRFullPointerField(
            NDRPacketField(
                "pbstrMachineGroupXml", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB
            )
        ),
        NDRFullPointerField(
            NDRPacketField(
                "pbstrConfigurationXmls", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB
            )
        ),
        NDRIntField("status", 0),
    ]


IRESOURCEMANAGER2_OPNUMS = {
    0: DceRpcOp(ExportObjects_Request, ExportObjects_Response),
    1: DceRpcOp(GetImportConflicts_Request, GetImportConflicts_Response),
    2: DceRpcOp(ImportXml_Request, ImportXml_Response),
    3: DceRpcOp(ExportXml_Request, ExportXml_Response),
}
register_com_interface(
    name="IResourceManager2",
    uuid=uuid.UUID("2A3EB639-D134-422d-90D8-AAA1B5216202"),
    opnums=IRESOURCEMANAGER2_OPNUMS,
)


class GetRemoteUserCategories_Request(NDRPacket):
    fields_desc = []


class GetRemoteUserCategories_Response(NDRPacket):
    fields_desc = [
        NDRFullPointerField(
            NDRPacketField(
                "pbstrRemoteUserCategoriesInfo", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB
            )
        ),
        NDRIntField("status", 0),
    ]


class SetRemoteUserCategories_Request(NDRPacket):
    fields_desc = [
        NDRPacketField(
            "bstrRemoteUserCategoriesInfo", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB
        )
    ]


class SetRemoteUserCategories_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class RefreshRemoteSessionWeights_Request(NDRPacket):
    fields_desc = [
        NDRPacketField(
            "bstrTaregetUserSessions", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB
        ),
        NDRSignedIntField("bUpdateAll", 0),
    ]


class RefreshRemoteSessionWeights_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


IWRMREMOTESESSIONMGMT_OPNUMS = {
    0: DceRpcOp(GetRemoteUserCategories_Request, GetRemoteUserCategories_Response),
    1: DceRpcOp(SetRemoteUserCategories_Request, SetRemoteUserCategories_Response),
    2: DceRpcOp(
        RefreshRemoteSessionWeights_Request, RefreshRemoteSessionWeights_Response
    ),
}
register_com_interface(
    name="IWRMRemoteSessionMgmt",
    uuid=uuid.UUID("FC910418-55CA-45ef-B264-83D4CE7D30E0"),
    opnums=IWRMREMOTESESSIONMGMT_OPNUMS,
)



### ms-comt.idl
"""
RPC definitions for the following interfaces:
- IGetTrackingData (v0.0): B60040E0-BCF3-11D1-861D-0080C729264D
- IComTrackingInfoEvents (v0.0): 4E6CDCC9-FB25-4FD5-9CC5-C9F4B6559CEC
- IProcessDump (v0.0): 23C9DD26-2355-4FE2-84DE-F779A238ADBD
This file is auto-generated by midl-to-scapy, do not modify.
"""

import uuid

from scapy.fields import StrFixedLenField, StrFixedLenFieldUtf16
from scapy.layers.dcerpc import (
    NDRPacket,
    DceRpcOp,
    NDRConfPacketListField,
    NDRConfStrLenField,
    NDRConfStrLenFieldUtf16,
    NDRFullPointerField,
    NDRIntField,
    NDRPacketField,
    NDRShortField,
    register_com_interface,
)


class ContainerStatistics(NDRPacket):
    ALIGNMENT = (4, 4)
    fields_desc = [
        NDRIntField("cCalls", 0),
        NDRIntField("cComponentInstances", 0),
        NDRIntField("cComponents", 0),
        NDRIntField("cCallsPerSecond", 0),
    ]


class ContainerData(NDRPacket):
    ALIGNMENT = (4, 4)
    fields_desc = [
        NDRIntField("dwLegacyId", 0),
        StrFixedLenFieldUtf16("wszApplicationIdentifier", "", length=40 * 2),
        NDRIntField("dwProcessId", 0),
        NDRPacketField("statistics", ContainerStatistics(), ContainerStatistics),
    ]


class GetContainerData_Request(NDRPacket):
    fields_desc = []


class GetContainerData_Response(NDRPacket):
    fields_desc = [
        NDRIntField("nContainers", None, size_of="aContainerData"),
        NDRConfPacketListField(
            "aContainerData",
            [],
            ContainerData,
            size_is=lambda pkt: pkt.nContainers,
            ptr_pack=True,
        ),
        NDRIntField("status", 0),
    ]


class GUID(NDRPacket):
    ALIGNMENT = (4, 4)
    fields_desc = [
        NDRIntField("Data1", 0),
        NDRShortField("Data2", 0),
        NDRShortField("Data3", 0),
        StrFixedLenField("Data4", "", length=8),
    ]


class ComponentData(NDRPacket):
    ALIGNMENT = (4, 4)
    fields_desc = [
        NDRPacketField("clsid", GUID(), GUID),
        NDRIntField("cTotalReferences", 0),
        NDRIntField("cBoundReferences", 0),
        NDRIntField("cPooledInstances", 0),
        NDRIntField("cInstancesInCall", 0),
        NDRIntField("dwResponseTime", 0),
        NDRIntField("cCallsCompleted", 0),
        NDRIntField("cCallsFailed", 0),
    ]


class GetComponentDataByContainer_Request(NDRPacket):
    fields_desc = [NDRIntField("idContainer", 0)]


class GetComponentDataByContainer_Response(NDRPacket):
    fields_desc = [
        NDRIntField("nComponents", None, size_of="aComponentData"),
        NDRConfPacketListField(
            "aComponentData",
            [],
            ComponentData,
            size_is=lambda pkt: pkt.nComponents,
            ptr_pack=True,
        ),
        NDRIntField("status", 0),
    ]


class GetComponentDataByContainerAndCLSID_Request(NDRPacket):
    fields_desc = [NDRIntField("idContainer", 0), NDRPacketField("clsid", GUID(), GUID)]


class GetComponentDataByContainerAndCLSID_Response(NDRPacket):
    fields_desc = [
        NDRFullPointerField(
            NDRPacketField("ppComponentData", ComponentData(), ComponentData)
        ),
        NDRIntField("status", 0),
    ]


IGETTRACKINGDATA_OPNUMS = {  # 0: Opnum3NotUsedOnWire,
    1: DceRpcOp(GetContainerData_Request, GetContainerData_Response),
    2: DceRpcOp(
        GetComponentDataByContainer_Request, GetComponentDataByContainer_Response
    ),
    3: DceRpcOp(
        GetComponentDataByContainerAndCLSID_Request,
        GetComponentDataByContainerAndCLSID_Response,
    ),
    # 4: Opnum7NotUsedOnWire
}
register_com_interface(
    name="IGetTrackingData",
    uuid=uuid.UUID("B60040E0-BCF3-11D1-861D-0080C729264D"),
    opnums=IGETTRACKINGDATA_OPNUMS,
)


class MInterfacePointer(NDRPacket):
    ALIGNMENT = (4, 8)
    DEPORTED_CONFORMANTS = ["abData"]
    fields_desc = [
        NDRIntField("ulCntData", None, size_of="abData"),
        NDRConfStrLenField(
            "abData", "", size_is=lambda pkt: pkt.ulCntData, conformant_in_struct=True
        ),
    ]


class OnNewTrackingInfo_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("pToplevelCollection", MInterfacePointer(), MInterfacePointer)
    ]


class OnNewTrackingInfo_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


ICOMTRACKINGINFOEVENTS_OPNUMS = {
    0: DceRpcOp(OnNewTrackingInfo_Request, OnNewTrackingInfo_Response)
}
register_com_interface(
    name="IComTrackingInfoEvents",
    uuid=uuid.UUID("4E6CDCC9-FB25-4FD5-9CC5-C9F4B6559CEC"),
    opnums=ICOMTRACKINGINFOEVENTS_OPNUMS,
)


class IsSupported_Request(NDRPacket):
    fields_desc = []


class IsSupported_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class FLAGGED_WORD_BLOB(NDRPacket):
    ALIGNMENT = (4, 8)
    DEPORTED_CONFORMANTS = ["asData"]
    fields_desc = [
        NDRIntField("cBytes", 0),
        NDRIntField("clSize", None, size_of="asData"),
        NDRConfStrLenFieldUtf16(
            "asData", "", size_is=lambda pkt: pkt.clSize, conformant_in_struct=True
        ),
    ]


class DumpProcess_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("bstrContainerID", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB),
        NDRPacketField("bstrDirectory", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB),
        NDRIntField("dwMaxFiles", 0),
    ]


class DumpProcess_Response(NDRPacket):
    fields_desc = [
        NDRFullPointerField(
            NDRPacketField("pbstrDumpFile", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB)
        ),
        NDRIntField("status", 0),
    ]


IPROCESSDUMP_OPNUMS = {
    0: DceRpcOp(IsSupported_Request, IsSupported_Response),
    1: DceRpcOp(DumpProcess_Request, DumpProcess_Response),
}
register_com_interface(
    name="IProcessDump",
    uuid=uuid.UUID("23C9DD26-2355-4FE2-84DE-F779A238ADBD"),
    opnums=IPROCESSDUMP_OPNUMS,
)



### ms-msrp_msgsvcsend.idl
"""
RPC definitions for the following interfaces:
- msgsvcsend (v1.0): 5a7b91f8-ff00-11d0-a9b2-00c04fb6e6fc
This file is auto-generated by midl-to-scapy, do not modify.
"""

import uuid


from scapy.layers.dcerpc import (
    NDRPacket,
    DceRpcOp,
    NDRConfVarStrNullField,
    NDRIntField,
    register_dcerpc_interface,
)


class NetrSendMessage_Request(NDRPacket):
    fields_desc = [
        NDRConfVarStrNullField("From", ""),
        NDRConfVarStrNullField("To", ""),
        NDRConfVarStrNullField("Text", ""),
    ]


class NetrSendMessage_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


MSGSVCSEND_OPNUMS = {0: DceRpcOp(NetrSendMessage_Request, NetrSendMessage_Response)}
register_dcerpc_interface(
    name="msgsvcsend",
    uuid=uuid.UUID("5a7b91f8-ff00-11d0-a9b2-00c04fb6e6fc"),
    version="1.0",
    opnums=MSGSVCSEND_OPNUMS,
)



### ms-tsgu.idl
"""
RPC definitions for the following interfaces:
- TsProxyRpcInterface (v1.3): 44e265dd-7daf-42cd-8560-3cdb6e7a2729
This file is auto-generated by midl-to-scapy, do not modify.
"""

import uuid

from scapy.fields import StrFixedLenField
from scapy.layers.dcerpc import (
    NDRPacket,
    DceRpcOp,
    NDRConfFieldListField,
    NDRConfPacketListField,
    NDRConfStrLenField,
    NDRConfStrLenFieldUtf16,
    NDRConfVarStrNullField,
    NDRConfVarStrNullFieldUtf16,
    NDRContextHandle,
    NDRFullPointerField,
    NDRIntField,
    NDRLongField,
    NDRPacketField,
    NDRShortField,
    NDRSignedIntField,
    NDRUnionField,
    register_dcerpc_interface,
)


class PTSG_PACKET_HEADER(NDRPacket):
    ALIGNMENT = (2, 2)
    fields_desc = [NDRShortField("ComponentId", 0), NDRShortField("PacketId", 0)]


class TSG_PACKET_HEADER(NDRPacket):
    ALIGNMENT = (2, 2)
    fields_desc = [NDRShortField("ComponentId", 0), NDRShortField("PacketId", 0)]


class TSG_CAPABILITY_NAP(NDRPacket):
    ALIGNMENT = (4, 4)
    fields_desc = [NDRIntField("capabilities", 0)]


class PTSG_PACKET_CAPABILITIES(NDRPacket):
    ALIGNMENT = (4, 4)
    fields_desc = [
        NDRIntField("capabilityType", 0),
        NDRUnionField(
            [
                (
                    NDRPacketField(
                        "TSGPacket", TSG_CAPABILITY_NAP(), TSG_CAPABILITY_NAP
                    ),
                    (
                        (lambda pkt: getattr(pkt, "capabilityType", None) == 1),
                        (lambda _, val: val.tag == 1),
                    ),
                )
            ],
            StrFixedLenField("TSGPacket", "", length=0),
            align=(4, 4),
            switch_fmt=("L", "L"),
        ),
    ]


class PTSG_PACKET_VERSIONCAPS(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRPacketField("tsgHeader", TSG_PACKET_HEADER(), TSG_PACKET_HEADER),
        NDRFullPointerField(
            NDRConfPacketListField(
                "TSGCaps",
                [PTSG_PACKET_CAPABILITIES()],
                PTSG_PACKET_CAPABILITIES,
                size_is=lambda pkt: pkt.numCapabilities,
            ),
            deferred=True,
        ),
        NDRIntField("numCapabilities", None, size_of="TSGCaps"),
        NDRShortField("majorVersion", 0),
        NDRShortField("minorVersion", 0),
        NDRShortField("quarantineCapabilities", 0),
    ]


class PTSG_PACKET_QUARCONFIGREQUEST(NDRPacket):
    ALIGNMENT = (4, 4)
    fields_desc = [NDRIntField("flags", 0)]


class PTSG_PACKET_QUARREQUEST(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRIntField("flags", 0),
        NDRFullPointerField(
            NDRConfStrLenFieldUtf16(
                "machineName", "", size_is=lambda pkt: pkt.nameLength
            ),
            deferred=True,
        ),
        NDRIntField("nameLength", None, size_of="machineName"),
        NDRFullPointerField(
            NDRConfStrLenField("data", "", size_is=lambda pkt: pkt.dataLen),
            deferred=True,
        ),
        NDRIntField("dataLen", None, size_of="data"),
    ]


class TSG_REDIRECTION_FLAGS(NDRPacket):
    ALIGNMENT = (4, 4)
    fields_desc = [
        NDRSignedIntField("enableAllRedirections", 0),
        NDRSignedIntField("disableAllRedirections", 0),
        NDRSignedIntField("driveRedirectionDisabled", 0),
        NDRSignedIntField("printerRedirectionDisabled", 0),
        NDRSignedIntField("portRedirectionDisabled", 0),
        NDRSignedIntField("reserved", 0),
        NDRSignedIntField("clipboardRedirectionDisabled", 0),
        NDRSignedIntField("pnpRedirectionDisabled", 0),
    ]


class PTSG_PACKET_RESPONSE(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRIntField("flags", 0),
        NDRIntField("reserved", 0),
        NDRFullPointerField(
            NDRConfStrLenField(
                "responseData", "", size_is=lambda pkt: pkt.responseDataLen
            ),
            deferred=True,
        ),
        NDRIntField("responseDataLen", None, size_of="responseData"),
        NDRPacketField(
            "redirectionFlags", TSG_REDIRECTION_FLAGS(), TSG_REDIRECTION_FLAGS
        ),
    ]


class GUID(NDRPacket):
    ALIGNMENT = (4, 4)
    fields_desc = [
        NDRIntField("Data1", 0),
        NDRShortField("Data2", 0),
        NDRShortField("Data3", 0),
        StrFixedLenField("Data4", "", length=8),
    ]


class PTSG_PACKET_QUARENC_RESPONSE(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRIntField("flags", 0),
        NDRIntField("certChainLen", None, size_of="certChainData"),
        NDRFullPointerField(
            NDRConfStrLenFieldUtf16(
                "certChainData", "", size_is=lambda pkt: pkt.certChainLen
            ),
            deferred=True,
        ),
        NDRPacketField("nonce", GUID(), GUID),
        NDRFullPointerField(
            NDRPacketField(
                "versionCaps", PTSG_PACKET_VERSIONCAPS(), PTSG_PACKET_VERSIONCAPS
            ),
            deferred=True,
        ),
    ]


class TSG_PACKET_QUARENC_RESPONSE(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRIntField("flags", 0),
        NDRIntField("certChainLen", None, size_of="certChainData"),
        NDRFullPointerField(
            NDRConfStrLenFieldUtf16(
                "certChainData", "", size_is=lambda pkt: pkt.certChainLen
            ),
            deferred=True,
        ),
        NDRPacketField("nonce", GUID(), GUID),
        NDRFullPointerField(
            NDRPacketField(
                "versionCaps", PTSG_PACKET_VERSIONCAPS(), PTSG_PACKET_VERSIONCAPS
            ),
            deferred=True,
        ),
    ]


class PTSG_PACKET_STRING_MESSAGE(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRSignedIntField("isDisplayMandatory", 0),
        NDRSignedIntField("isConsentMandatory", 0),
        NDRIntField("msgBytes", None, size_of="msgBuffer"),
        NDRFullPointerField(
            NDRConfStrLenFieldUtf16("msgBuffer", "", size_is=lambda pkt: pkt.msgBytes),
            deferred=True,
        ),
    ]


class PTSG_PACKET_REAUTH_MESSAGE(NDRPacket):
    ALIGNMENT = (8, 8)
    fields_desc = [NDRLongField("tunnelContext", 0)]


class TSG_PACKET_MSG_RESPONSE(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRIntField("msgID", 0),
        NDRIntField("msgType", 0),
        NDRSignedIntField("isMsgPresent", 0),
        NDRUnionField(
            [
                (
                    NDRFullPointerField(
                        NDRPacketField(
                            "messagePacket",
                            PTSG_PACKET_STRING_MESSAGE(),
                            PTSG_PACKET_STRING_MESSAGE,
                        ),
                        deferred=True,
                    ),
                    (
                        (lambda pkt: getattr(pkt, "msgType", None) == 1),
                        (lambda _, val: val.tag == 1),
                    ),
                ),
                (
                    NDRFullPointerField(
                        NDRPacketField(
                            "messagePacket",
                            PTSG_PACKET_STRING_MESSAGE(),
                            PTSG_PACKET_STRING_MESSAGE,
                        ),
                        deferred=True,
                    ),
                    (
                        (lambda pkt: getattr(pkt, "msgType", None) == 2),
                        (lambda _, val: val.tag == 2),
                    ),
                ),
                (
                    NDRFullPointerField(
                        NDRPacketField(
                            "messagePacket",
                            PTSG_PACKET_REAUTH_MESSAGE(),
                            PTSG_PACKET_REAUTH_MESSAGE,
                        ),
                        deferred=True,
                    ),
                    (
                        (lambda pkt: getattr(pkt, "msgType", None) == 3),
                        (lambda _, val: val.tag == 3),
                    ),
                ),
            ],
            StrFixedLenField("messagePacket", "", length=0),
            align=(4, 8),
            switch_fmt=("L", "L"),
        ),
    ]


class PTSG_PACKET_CAPS_RESPONSE(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRPacketField(
            "pktQuarEncResponse",
            TSG_PACKET_QUARENC_RESPONSE(),
            TSG_PACKET_QUARENC_RESPONSE,
        ),
        NDRPacketField(
            "pktConsentMessage", TSG_PACKET_MSG_RESPONSE(), TSG_PACKET_MSG_RESPONSE
        ),
    ]


class PTSG_PACKET_MSG_REQUEST(NDRPacket):
    ALIGNMENT = (4, 4)
    fields_desc = [NDRIntField("maxMessagesPerBatch", 0)]


class PTSG_PACKET_MSG_RESPONSE(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRIntField("msgID", 0),
        NDRIntField("msgType", 0),
        NDRSignedIntField("isMsgPresent", 0),
        NDRUnionField(
            [
                (
                    NDRFullPointerField(
                        NDRPacketField(
                            "messagePacket",
                            PTSG_PACKET_STRING_MESSAGE(),
                            PTSG_PACKET_STRING_MESSAGE,
                        ),
                        deferred=True,
                    ),
                    (
                        (lambda pkt: getattr(pkt, "msgType", None) == 1),
                        (lambda _, val: val.tag == 1),
                    ),
                ),
                (
                    NDRFullPointerField(
                        NDRPacketField(
                            "messagePacket",
                            PTSG_PACKET_STRING_MESSAGE(),
                            PTSG_PACKET_STRING_MESSAGE,
                        ),
                        deferred=True,
                    ),
                    (
                        (lambda pkt: getattr(pkt, "msgType", None) == 2),
                        (lambda _, val: val.tag == 2),
                    ),
                ),
                (
                    NDRFullPointerField(
                        NDRPacketField(
                            "messagePacket",
                            PTSG_PACKET_REAUTH_MESSAGE(),
                            PTSG_PACKET_REAUTH_MESSAGE,
                        ),
                        deferred=True,
                    ),
                    (
                        (lambda pkt: getattr(pkt, "msgType", None) == 3),
                        (lambda _, val: val.tag == 3),
                    ),
                ),
            ],
            StrFixedLenField("messagePacket", "", length=0),
            align=(4, 8),
            switch_fmt=("L", "L"),
        ),
    ]


class TSG_PACKET_VERSIONCAPS(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRPacketField("tsgHeader", TSG_PACKET_HEADER(), TSG_PACKET_HEADER),
        NDRFullPointerField(
            NDRConfPacketListField(
                "TSGCaps",
                [PTSG_PACKET_CAPABILITIES()],
                PTSG_PACKET_CAPABILITIES,
                size_is=lambda pkt: pkt.numCapabilities,
            ),
            deferred=True,
        ),
        NDRIntField("numCapabilities", None, size_of="TSGCaps"),
        NDRShortField("majorVersion", 0),
        NDRShortField("minorVersion", 0),
        NDRShortField("quarantineCapabilities", 0),
    ]


class PTSG_PACKET_AUTH(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRPacketField(
            "TSGVersionCaps", TSG_PACKET_VERSIONCAPS(), TSG_PACKET_VERSIONCAPS
        ),
        NDRIntField("cookieLen", None, size_of="cookie"),
        NDRFullPointerField(
            NDRConfStrLenField("cookie", "", size_is=lambda pkt: pkt.cookieLen),
            deferred=True,
        ),
    ]


class PTSG_PACKET_REAUTH(NDRPacket):
    ALIGNMENT = (8, 8)
    fields_desc = [
        NDRLongField("tunnelContext", 0),
        NDRIntField("packetId", 0),
        NDRUnionField(
            [
                (
                    NDRFullPointerField(
                        NDRPacketField(
                            "TSGInitialPacket",
                            PTSG_PACKET_VERSIONCAPS(),
                            PTSG_PACKET_VERSIONCAPS,
                        ),
                        deferred=True,
                    ),
                    (
                        (lambda pkt: getattr(pkt, "packetId", None) == 22083),
                        (lambda _, val: val.tag == 22083),
                    ),
                ),
                (
                    NDRFullPointerField(
                        NDRPacketField(
                            "TSGInitialPacket", PTSG_PACKET_AUTH(), PTSG_PACKET_AUTH
                        ),
                        deferred=True,
                    ),
                    (
                        (lambda pkt: getattr(pkt, "packetId", None) == 16468),
                        (lambda _, val: val.tag == 16468),
                    ),
                ),
            ],
            StrFixedLenField("TSGInitialPacket", "", length=0),
            align=(4, 8),
            switch_fmt=("L", "L"),
        ),
    ]


class PTSG_PACKET(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRIntField("packetId", 0),
        NDRUnionField(
            [
                (
                    NDRFullPointerField(
                        NDRPacketField(
                            "TSGPacket", PTSG_PACKET_HEADER(), PTSG_PACKET_HEADER
                        ),
                        deferred=True,
                    ),
                    (
                        (lambda pkt: getattr(pkt, "packetId", None) == 18500),
                        (lambda _, val: val.tag == 18500),
                    ),
                ),
                (
                    NDRFullPointerField(
                        NDRPacketField(
                            "TSGPacket",
                            PTSG_PACKET_VERSIONCAPS(),
                            PTSG_PACKET_VERSIONCAPS,
                        ),
                        deferred=True,
                    ),
                    (
                        (lambda pkt: getattr(pkt, "packetId", None) == 22083),
                        (lambda _, val: val.tag == 22083),
                    ),
                ),
                (
                    NDRFullPointerField(
                        NDRPacketField(
                            "TSGPacket",
                            PTSG_PACKET_QUARCONFIGREQUEST(),
                            PTSG_PACKET_QUARCONFIGREQUEST,
                        ),
                        deferred=True,
                    ),
                    (
                        (lambda pkt: getattr(pkt, "packetId", None) == 20803),
                        (lambda _, val: val.tag == 20803),
                    ),
                ),
                (
                    NDRFullPointerField(
                        NDRPacketField(
                            "TSGPacket",
                            PTSG_PACKET_QUARREQUEST(),
                            PTSG_PACKET_QUARREQUEST,
                        ),
                        deferred=True,
                    ),
                    (
                        (lambda pkt: getattr(pkt, "packetId", None) == 20818),
                        (lambda _, val: val.tag == 20818),
                    ),
                ),
                (
                    NDRFullPointerField(
                        NDRPacketField(
                            "TSGPacket", PTSG_PACKET_RESPONSE(), PTSG_PACKET_RESPONSE
                        ),
                        deferred=True,
                    ),
                    (
                        (lambda pkt: getattr(pkt, "packetId", None) == 20562),
                        (lambda _, val: val.tag == 20562),
                    ),
                ),
                (
                    NDRFullPointerField(
                        NDRPacketField(
                            "TSGPacket",
                            PTSG_PACKET_QUARENC_RESPONSE(),
                            PTSG_PACKET_QUARENC_RESPONSE,
                        ),
                        deferred=True,
                    ),
                    (
                        (lambda pkt: getattr(pkt, "packetId", None) == 17746),
                        (lambda _, val: val.tag == 17746),
                    ),
                ),
                (
                    NDRFullPointerField(
                        NDRPacketField(
                            "TSGPacket",
                            PTSG_PACKET_CAPS_RESPONSE(),
                            PTSG_PACKET_CAPS_RESPONSE,
                        ),
                        deferred=True,
                    ),
                    (
                        (lambda pkt: getattr(pkt, "packetId", None) == 17232),
                        (lambda _, val: val.tag == 17232),
                    ),
                ),
                (
                    NDRFullPointerField(
                        NDRPacketField(
                            "TSGPacket",
                            PTSG_PACKET_MSG_REQUEST(),
                            PTSG_PACKET_MSG_REQUEST,
                        ),
                        deferred=True,
                    ),
                    (
                        (lambda pkt: getattr(pkt, "packetId", None) == 18258),
                        (lambda _, val: val.tag == 18258),
                    ),
                ),
                (
                    NDRFullPointerField(
                        NDRPacketField(
                            "TSGPacket",
                            PTSG_PACKET_MSG_RESPONSE(),
                            PTSG_PACKET_MSG_RESPONSE,
                        ),
                        deferred=True,
                    ),
                    (
                        (lambda pkt: getattr(pkt, "packetId", None) == 18256),
                        (lambda _, val: val.tag == 18256),
                    ),
                ),
                (
                    NDRFullPointerField(
                        NDRPacketField(
                            "TSGPacket", PTSG_PACKET_AUTH(), PTSG_PACKET_AUTH
                        ),
                        deferred=True,
                    ),
                    (
                        (lambda pkt: getattr(pkt, "packetId", None) == 16468),
                        (lambda _, val: val.tag == 16468),
                    ),
                ),
                (
                    NDRFullPointerField(
                        NDRPacketField(
                            "TSGPacket", PTSG_PACKET_REAUTH(), PTSG_PACKET_REAUTH
                        ),
                        deferred=True,
                    ),
                    (
                        (lambda pkt: getattr(pkt, "packetId", None) == 21072),
                        (lambda _, val: val.tag == 21072),
                    ),
                ),
            ],
            StrFixedLenField("TSGPacket", "", length=0),
            align=(4, 8),
            switch_fmt=("L", "L"),
        ),
    ]


class TsProxyCreateTunnel_Request(NDRPacket):
    fields_desc = [NDRPacketField("TSGPacket", PTSG_PACKET(), PTSG_PACKET)]


class TsProxyCreateTunnel_Response(NDRPacket):
    fields_desc = [
        NDRFullPointerField(
            NDRPacketField("TSGPacketResponse", PTSG_PACKET(), PTSG_PACKET)
        ),
        NDRPacketField("tunnelContext", NDRContextHandle(), NDRContextHandle),
        NDRIntField("tunnelId", 0),
        NDRIntField("status", 0),
    ]


class TsProxyAuthorizeTunnel_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("tunnelContext", NDRContextHandle(), NDRContextHandle),
        NDRPacketField("TSGPacket", PTSG_PACKET(), PTSG_PACKET),
    ]


class TsProxyAuthorizeTunnel_Response(NDRPacket):
    fields_desc = [
        NDRFullPointerField(
            NDRPacketField("TSGPacketResponse", PTSG_PACKET(), PTSG_PACKET)
        ),
        NDRIntField("status", 0),
    ]


class TsProxyMakeTunnelCall_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("tunnelContext", NDRContextHandle(), NDRContextHandle),
        NDRIntField("procId", 0),
        NDRPacketField("TSGPacket", PTSG_PACKET(), PTSG_PACKET),
    ]


class TsProxyMakeTunnelCall_Response(NDRPacket):
    fields_desc = [
        NDRFullPointerField(
            NDRPacketField("TSGPacketResponse", PTSG_PACKET(), PTSG_PACKET)
        ),
        NDRIntField("status", 0),
    ]


class PTSENDPOINTINFO(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRFullPointerField(
            NDRConfFieldListField(
                "resourceName",
                [],
                NDRFullPointerField(
                    NDRConfVarStrNullFieldUtf16("resourceName", ""), deferred=True
                ),
                size_is=lambda pkt: pkt.numResourceNames,
            ),
            deferred=True,
        ),
        NDRIntField("numResourceNames", None, size_of="resourceName"),
        NDRFullPointerField(
            NDRConfFieldListField(
                "alternateResourceNames",
                [],
                NDRFullPointerField(
                    NDRConfVarStrNullFieldUtf16("alternateResourceNames", ""),
                    deferred=True,
                ),
                size_is=lambda pkt: pkt.numAlternateResourceNames,
            ),
            deferred=True,
        ),
        NDRShortField(
            "numAlternateResourceNames", None, size_of="alternateResourceNames"
        ),
        NDRIntField("Port", 0),
    ]


class TsProxyCreateChannel_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("tunnelContext", NDRContextHandle(), NDRContextHandle),
        NDRPacketField("tsEndPointInfo", PTSENDPOINTINFO(), PTSENDPOINTINFO),
    ]


class TsProxyCreateChannel_Response(NDRPacket):
    fields_desc = [
        NDRPacketField("channelContext", NDRContextHandle(), NDRContextHandle),
        NDRIntField("channelId", 0),
        NDRIntField("status", 0),
    ]


class TsProxyCloseChannel_Request(NDRPacket):
    fields_desc = [NDRPacketField("context", NDRContextHandle(), NDRContextHandle)]


class TsProxyCloseChannel_Response(NDRPacket):
    fields_desc = [
        NDRPacketField("context", NDRContextHandle(), NDRContextHandle),
        NDRIntField("status", 0),
    ]


class TsProxyCloseTunnel_Request(NDRPacket):
    fields_desc = [NDRPacketField("context", NDRContextHandle(), NDRContextHandle)]


class TsProxyCloseTunnel_Response(NDRPacket):
    fields_desc = [
        NDRPacketField("context", NDRContextHandle(), NDRContextHandle),
        NDRIntField("status", 0),
    ]


class TsProxySetupReceivePipe_Request(NDRPacket):
    fields_desc = [NDRConfStrLenField("pRpcMessage", "", size_is=lambda pkt: 32767)]


class TsProxySetupReceivePipe_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class TsProxySendToServer_Request(NDRPacket):
    fields_desc = [NDRConfStrLenField("pRpcMessage", "", size_is=lambda pkt: 32767)]


class TsProxySendToServer_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


TSPROXYRPCINTERFACE_OPNUMS = {  # 0: Opnum0NotUsedOnWire,
    1: DceRpcOp(TsProxyCreateTunnel_Request, TsProxyCreateTunnel_Response),
    2: DceRpcOp(TsProxyAuthorizeTunnel_Request, TsProxyAuthorizeTunnel_Response),
    3: DceRpcOp(TsProxyMakeTunnelCall_Request, TsProxyMakeTunnelCall_Response),
    4: DceRpcOp(TsProxyCreateChannel_Request, TsProxyCreateChannel_Response),
    # 5: Opnum5NotUsedOnWire,
    6: DceRpcOp(TsProxyCloseChannel_Request, TsProxyCloseChannel_Response),
    7: DceRpcOp(TsProxyCloseTunnel_Request, TsProxyCloseTunnel_Response),
    8: DceRpcOp(TsProxySetupReceivePipe_Request, TsProxySetupReceivePipe_Response),
    9: DceRpcOp(TsProxySendToServer_Request, TsProxySendToServer_Response),
}
register_dcerpc_interface(
    name="TsProxyRpcInterface",
    uuid=uuid.UUID("44e265dd-7daf-42cd-8560-3cdb6e7a2729"),
    version="1.3",
    opnums=TSPROXYRPCINTERFACE_OPNUMS,
)
